/*
  Add stored procedures.
*/
/*
  Convert nvarchar delimited by specified delimiter into number table.
*/
CREATE FUNCTION ConvertDelimitedIdsToTable(@delimString NVARCHAR(MAX), @delim CHAR(1)) 
RETURNS @result TABLE (ID BIGINT) 
AS 
BEGIN
  DECLARE @xml xml
  SET @xml = N'<root><r>' + replace(@delimString, @delim, '</r><r>') + '</r></root>'

  INSERT INTO @result(ID)
  SELECT
    r.value('.','BIGINT') as ID
  FROM @xml.nodes('//root/r') as records(r)
  RETURN
END;
GO

CREATE TYPE PA_TABLETYPE_BigInt AS TABLE ( VALUE_1 BIGINT );
CREATE TYPE PA_TABLETYPE_NVarCharMax AS TABLE ( VALUE_1 NVarChar(MAX) );

CREATE TABLE PA_USER (
  USR_ID INT IDENTITY NOT NULL,
  USR_LOGON_NAME NVARCHAR(440),
  USR_PASSWORD_HASH NVARCHAR(255),
  USR_EMAIL NVARCHAR(MAX),
  USR_DESCRIPTION NVARCHAR(MAX),
  USR_FULL_NAME NVARCHAR(MAX),
  USR_FLAGS INT,
  USR_JOIN_TOKENS NVARCHAR(MAX),
  USR_LOGON_MESSAGE NVARCHAR(MAX),
  USR_INACTIVE_ACCOUNT_MESSAGE NVARCHAR(MAX),
  USR_LAST_LOGIN_DATE DATETIME,
  USR_LOGIN_ATTEMPTS_COUNT INT,
  USR_BLOCKED_DATE DATETIME
);

ALTER TABLE PA_USER ADD PRIMARY KEY (USR_ID);
ALTER TABLE PA_USER ADD CONSTRAINT PA_USER_UNIQUE_LOGON_NAME UNIQUE (USR_LOGON_NAME);

CREATE TABLE PA_USER_GROUP (
  USG_ID BIGINT IDENTITY NOT NULL,
  USG_GROUP_ID INT,
  USG_MEMBER_ID INT,
  USG_ROLE_ID INT
);

ALTER TABLE PA_USER_GROUP ADD PRIMARY KEY (USG_ID);
ALTER TABLE PA_USER_GROUP ADD CONSTRAINT PA_USER_GROUP_GROUP FOREIGN KEY (USG_GROUP_ID) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_USER_GROUP ADD CONSTRAINT PA_USER_GROUP_MEMBER FOREIGN KEY (USG_MEMBER_ID) REFERENCES PA_USER(USR_ID);
CREATE INDEX PA_USER_GROUP_GROUP_ID ON PA_USER_GROUP (USG_GROUP_ID);
CREATE INDEX PA_USER_GROUP_MEMBER_ID ON PA_USER_GROUP (USG_MEMBER_ID);

CREATE TABLE PA_PROJECT (
  PRO_ID BIGINT IDENTITY NOT NULL PRIMARY KEY,
  PRO_PARENT_ID BIGINT,
  PRO_NAME NVARCHAR(440),
  PRO_DESCRIPTION NVARCHAR(MAX),
  PRO_CREATED_DATE DATETIME,
  PRO_CREATED_BY INT,
  PRO_LAST_MODIFIED_DATE DATETIME,
  PRO_LAST_MODIFIED_BY INT,
  PRO_DELETED_DATE DATETIME,
  PRO_DELETED_BY INT,
  PRO_FLAGS INT
);
ALTER TABLE PA_PROJECT ADD CONSTRAINT PA_PROJECT_PARENT_ID FOREIGN KEY (PRO_PARENT_ID) REFERENCES PA_PROJECT(PRO_ID);
ALTER TABLE PA_PROJECT ADD CONSTRAINT PA_PROJECT_CREATED_BY FOREIGN KEY (PRO_CREATED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_PROJECT ADD CONSTRAINT PA_PROJECT_LAST_MODIFIED_BY FOREIGN KEY (PRO_LAST_MODIFIED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_PROJECT ADD CONSTRAINT PA_PROJECT_DELETED_BY FOREIGN KEY (PRO_DELETED_BY) REFERENCES PA_USER(USR_ID);

CREATE TABLE PA_MODEL (
  MOD_ID BIGINT IDENTITY NOT NULL,
  MOD_PARENT_ID BIGINT,
  MOD_PROJECT_ID BIGINT,
  MOD_NAME NVARCHAR(440),
  MOD_DESCRIPTION NVARCHAR(MAX),
  MOD_CREATED_DATE DATETIME,
  MOD_CREATED_BY INT,
  MOD_LAST_MODIFIED_DATE DATETIME,
  MOD_LAST_MODIFIED_BY INT,
  MOD_LAST_IMPORT_DATE DATETIME,
  MOD_LAST_IMPORT_BY INT,
  MOD_ARCHIVED_DATE DATETIME,
  MOD_ARCHIVED_BY INT,
  MOD_REPLACED_BY_MODEL_ID BIGINT,
  MOD_FLAGS INT,
  MOD_TYPE INT,
  MOD_NUM_VIEWS BIGINT,
  MOD_NUM_REPORTS BIGINT,
  MOD_NUM_ANALYSES BIGINT,
  MOD_NUM_EVENTS BIGINT,
  MOD_NUM_ACTIVITIES BIGINT,
  MOD_NUM_CASES BIGINT,
  MOD_NUM_EVENT_ATTRIBUTE_TYPES BIGINT,
  MOD_NUM_CASE_ATTRIBUTE_TYPES BIGINT,
  MOD_MAX_NUM_EVENTS BIGINT,
  MOD_MAX_NUM_EVENT_ATTRIBUTE_TYPES BIGINT,
  MOD_MAX_NUM_CASE_ATTRIBUTE_TYPES BIGINT,
  MOD_COMPATIBILITY_VERSION INT,
  MOD_CONFIGURATION NVARCHAR(MAX)
);
ALTER TABLE PA_MODEL ADD PRIMARY KEY (MOD_ID);
ALTER TABLE PA_MODEL ADD CONSTRAINT PA_MODEL_PROJECT FOREIGN KEY (MOD_PROJECT_ID) REFERENCES PA_PROJECT(PRO_ID);
ALTER TABLE PA_MODEL ADD CONSTRAINT PA_MODEL_CREATED_BY FOREIGN KEY (MOD_CREATED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_MODEL ADD CONSTRAINT PA_MODEL_LAST_MODIFIED_BY FOREIGN KEY (MOD_LAST_MODIFIED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_MODEL ADD CONSTRAINT PA_MODEL_REPLACED_BY_MODEL FOREIGN KEY 
  (MOD_REPLACED_BY_MODEL_ID) REFERENCES PA_MODEL (MOD_ID);
ALTER TABLE PA_MODEL ADD CONSTRAINT PA_MODEL_PARENT FOREIGN KEY 
  (MOD_PARENT_ID) REFERENCES PA_MODEL (MOD_ID);
CREATE INDEX PA_MODEL_REPLACED_BY_MODEL_ID ON PA_MODEL (MOD_REPLACED_BY_MODEL_ID);

CREATE TABLE PA_VIEW (
  VIE_ID BIGINT IDENTITY NOT NULL,
  VIE_PARENT_ID BIGINT,
  VIE_MODEL_ID BIGINT,
  VIE_NAME NVARCHAR(MAX),
  VIE_DESCRIPTION NVARCHAR(MAX),
  VIE_FLAGS INT,
  VIE_CREATED_DATE DATETIME,
  VIE_CREATED_BY INT,
  VIE_LAST_MODIFIED_DATE DATETIME,
  VIE_LAST_MODIFIED_BY INT,
  VIE_CASE_SAMPLE_SIZE BIGINT,
  VIE_PUBLISH_MODE INT,
  VIE_LAST_CACHE_ACCESS DATETIME,
  VIE_LAST_RECALCULATION_DATE DATETIME,
  VIE_RECALCULATION_MODE INT,
  VIE_NUM_ANALYSES BIGINT,
  VIE_NUM_REPORTS BIGINT,
  VIE_NUM_HIDDEN_EVENT_TYPES BIGINT,
  VIE_NUM_HIDDEN_CASES BIGINT,
  VIE_NUM_HIDDEN_VARIATIONS BIGINT,
  VIE_NUM_HIDDEN_CASE_ATTRIBUTES BIGINT, 
  VIE_NUM_HIDDEN_EVENT_ATTRIBUTES BIGINT, 
  VIE_HIDDEN_EVENT_TYPE_IDS NVARCHAR(MAX),
  VIE_HIDDEN_CASE_ATTRIBUTE_IDS NVARCHAR(MAX),
  VIE_HIDDEN_EVENT_ATTRIBUTE_IDS NVARCHAR(MAX),
  VIE_NUM_SAMPLED_CASES BIGINT,
  VIE_NUM_SAMPLED_EVENTS BIGINT,
  VIE_NUM_SAMPLED_EVENT_TYPES BIGINT,
  VIE_NUM_VALID_CASES BIGINT,
  VIE_CONFIGURATION NVARCHAR(MAX)
);
ALTER TABLE PA_VIEW ADD PRIMARY KEY (VIE_ID);
ALTER TABLE PA_VIEW ADD CONSTRAINT PA_VIEW_MODEL FOREIGN KEY
	(VIE_MODEL_ID) REFERENCES PA_MODEL(MOD_ID);
CREATE INDEX PA_VIEW_MODEL_ID ON PA_VIEW (VIE_MODEL_ID);
ALTER TABLE PA_VIEW ADD CONSTRAINT PA_VIEW_CREATED_BY FOREIGN KEY (VIE_CREATED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_VIEW ADD CONSTRAINT PA_VIEW_LAST_MODIFIED_BY FOREIGN KEY (VIE_LAST_MODIFIED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_VIEW ADD CONSTRAINT PA_VIEW_PARENT FOREIGN KEY (VIE_PARENT_ID) REFERENCES PA_VIEW(VIE_ID);

CREATE TABLE PA_VIEW_FILTER_RULE (
  VFR_ID BIGINT IDENTITY NOT NULL,
  VFR_MODEL_ID BIGINT,
  VFR_VIEW_ID BIGINT,
  VFR_PARENT_ID BIGINT,
  VFR_CREATED_DATE DATETIME,
  VFR_CREATED_BY INT,
  VFR_FLAGS INT,
  VFR_TYPE INT,
  VFR_SOURCE_ANALYSIS_TYPE INT,
  VFR_ORDER INT,
  VFR_VERSION INT,
  VFR_INHERITED_FROM_ID BIGINT
);

ALTER TABLE PA_VIEW_FILTER_RULE ADD PRIMARY KEY (VFR_ID);
ALTER TABLE PA_VIEW_FILTER_RULE ADD CONSTRAINT PA_VIEW_FILTER_RULE_MODEL FOREIGN KEY
	(VFR_MODEL_ID) REFERENCES PA_MODEL(MOD_ID);
ALTER TABLE PA_VIEW_FILTER_RULE ADD CONSTRAINT PA_VIEW_FILTER_RULE_VIEW FOREIGN KEY
	(VFR_VIEW_ID) REFERENCES PA_VIEW(VIE_ID);
ALTER TABLE PA_VIEW_FILTER_RULE ADD CONSTRAINT PA_VIEW_FILTER_RULE_CREATED_BY FOREIGN KEY (VFR_CREATED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_VIEW_FILTER_RULE ADD CONSTRAINT PA_VIEW_FILTER_RULE_PARENT FOREIGN KEY (VFR_PARENT_ID) REFERENCES PA_VIEW_FILTER_RULE(VFR_ID);

CREATE TABLE PA_VIEW_FILTER_RULE_PARAMETER (
  VFP_ID BIGINT IDENTITY NOT NULL,
  VFP_MODEL_ID BIGINT,
  VFP_FILTER_RULE_ID BIGINT,
  VFP_KEY NVARCHAR(440),
  VFP_VALUE NVARCHAR(MAX)
);

ALTER TABLE PA_VIEW_FILTER_RULE_PARAMETER ADD PRIMARY KEY (VFP_ID);
ALTER TABLE PA_VIEW_FILTER_RULE_PARAMETER ADD CONSTRAINT PA_VIEW_FILTER_RULE_PARAMETER_MODEL FOREIGN KEY
	(VFP_MODEL_ID) REFERENCES PA_MODEL(MOD_ID);
ALTER TABLE PA_VIEW_FILTER_RULE_PARAMETER ADD CONSTRAINT PA_VIEW_FILTER_RULE_PARAMETER_FILTER_RULE FOREIGN KEY
	(VFP_FILTER_RULE_ID) REFERENCES PA_VIEW_FILTER_RULE(VFR_ID);

CREATE INDEX PA_VIEW_FILTER_RULE_PARAMETER_MODEL_ID ON PA_VIEW_FILTER_RULE_PARAMETER (VFP_MODEL_ID);
CREATE INDEX PA_VIEW_FILTER_RULE_PARAMETER_FILTER_RULE_ID ON PA_VIEW_FILTER_RULE_PARAMETER (VFP_FILTER_RULE_ID);

CREATE TABLE PA_REPORT (
  REP_ID BIGINT IDENTITY NOT NULL,
  REP_MODEL_ID BIGINT,
  REP_VIEW_ID BIGINT,
  REP_NAME NVARCHAR(MAX),
  REP_SHEET_NAME NVARCHAR(MAX),
  REP_DESCRIPTION NVARCHAR(MAX),
  REP_FLAGS INT,
  REP_CREATED_DATE DATETIME,
  REP_CREATED_BY INT,
  REP_LAST_MODIFIED_DATE DATETIME,
  REP_LAST_MODIFIED_BY INT,
  REP_PUBLISH_MODE INT,
  REP_NUM_OPENS BIGINT
);

ALTER TABLE PA_REPORT ADD PRIMARY KEY (REP_ID);
ALTER TABLE PA_REPORT ADD CONSTRAINT PA_REPORT_MODEL FOREIGN KEY
	(REP_MODEL_ID) REFERENCES PA_MODEL(MOD_ID);
CREATE INDEX PA_REPORT_MODEL_ID ON PA_REPORT (REP_MODEL_ID);
ALTER TABLE PA_REPORT ADD CONSTRAINT PA_REPORT_VIEW FOREIGN KEY
	(REP_VIEW_ID) REFERENCES PA_VIEW(VIE_ID);
CREATE INDEX PA_REPORT_VIEW_ID ON PA_REPORT (REP_VIEW_ID);
ALTER TABLE PA_REPORT ADD CONSTRAINT PA_REPORT_CREATED_BY FOREIGN KEY (REP_CREATED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_REPORT ADD CONSTRAINT PA_REPORT_LAST_MODIFIED_BY FOREIGN KEY (REP_LAST_MODIFIED_BY) REFERENCES PA_USER(USR_ID);

CREATE TABLE PA_REPORT_SETTING (
  RES_ID BIGINT IDENTITY NOT NULL,
  RES_REPORT_ID BIGINT,
  RES_MODEL_ID BIGINT,
  RES_KEY NVARCHAR(440),
  RES_VALUE NVARCHAR(MAX)
);

ALTER TABLE PA_REPORT_SETTING ADD PRIMARY KEY (RES_ID);
ALTER TABLE PA_REPORT_SETTING ADD CONSTRAINT PA_REPORT_SETTING_MODEL FOREIGN KEY
	(RES_MODEL_ID) REFERENCES PA_MODEL(MOD_ID);
ALTER TABLE PA_REPORT_SETTING ADD CONSTRAINT PA_REPORT_SETTING_REPORT FOREIGN KEY
	(RES_REPORT_ID) REFERENCES PA_REPORT(REP_ID);

CREATE INDEX PA_REPORT_SETTING_MODEL_ID ON PA_REPORT_SETTING (RES_MODEL_ID);
CREATE INDEX PA_REPORT_SETTING_REPORT_ID ON PA_REPORT_SETTING (RES_REPORT_ID);

CREATE TABLE PA_ROLE (
  ROL_ID INT IDENTITY NOT NULL,
  ROL_NAME NVARCHAR(255),
  ROL_MAX_NUM_MODELS BIGINT,
  ROL_MODEL_MAX_NUM_EVENTS BIGINT,
  ROL_MODEL_MAX_NUM_CASE_ATTRIBUTE_TYPES BIGINT,
  ROL_MODEL_MAX_NUM_EVENT_ATTRIBUTE_TYPES BIGINT,
  ROL_FLAGS INT,
  ROL_MAX_NUM_INTEGRATION_TABLES BIGINT,
  ROL_INTEGRATION_TABLE_MAX_NUM_COLUMNS BIGINT,
  ROL_INTEGRATION_TABLE_MAX_NUM_ROWS BIGINT
);

ALTER TABLE PA_ROLE ADD PRIMARY KEY (ROL_ID);

CREATE TABLE PA_USER_ROLE (
  URO_ID INT IDENTITY NOT NULL,
  URO_PROJECT_ID BIGINT,
  URO_USER_ID INT,
  URO_ROLE_ID INT
);

ALTER TABLE PA_USER_ROLE ADD PRIMARY KEY (URO_ID);
ALTER TABLE PA_USER_ROLE ADD CONSTRAINT PA_USER_ROLE_PROJECT FOREIGN KEY (URO_PROJECT_ID) REFERENCES PA_PROJECT(PRO_ID);
ALTER TABLE PA_USER_ROLE ADD CONSTRAINT PA_USER_ROLE_USER FOREIGN KEY
	(URO_USER_ID) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_USER_ROLE ADD CONSTRAINT PA_USER_ROLE_ROLE FOREIGN KEY
	(URO_ROLE_ID) REFERENCES PA_ROLE(ROL_ID);
CREATE INDEX PA_USER_ROLE_PROJECT_ID ON PA_USER_ROLE (URO_PROJECT_ID);
CREATE INDEX PA_USER_ROLE_USER_ID ON PA_USER_ROLE (URO_USER_ID);
CREATE INDEX PA_USER_ROLE_ROLE_ID ON PA_USER_ROLE (URO_ROLE_ID);

CREATE TABLE PA_PERMISSION (
  PER_ID INT IDENTITY NOT NULL,
  PER_NAME NVARCHAR(440)
);

ALTER TABLE PA_PERMISSION ADD PRIMARY KEY (PER_ID);

CREATE TABLE PA_ROLE_PERMISSION (
  ROP_ID INT IDENTITY NOT NULL,
  ROP_ROLE_ID INT,
  ROP_PERMISSION_ID INT
);

ALTER TABLE PA_ROLE_PERMISSION ADD PRIMARY KEY (ROP_ID);
ALTER TABLE PA_ROLE_PERMISSION ADD CONSTRAINT PA_PA_ROLE_PERMISSION_ROLE FOREIGN KEY
	(ROP_ROLE_ID) REFERENCES PA_ROLE(ROL_ID);
ALTER TABLE PA_ROLE_PERMISSION ADD CONSTRAINT PA_PA_ROLE_PERMISSION_PERMISSION FOREIGN KEY
	(ROP_PERMISSION_ID) REFERENCES PA_PERMISSION(PER_ID);

CREATE TABLE PA_OPERATION_STATUS (
  OPE_ID BIGINT IDENTITY NOT NULL,
  OPE_NAME NTEXT,
  OPE_SESSION_ID UNIQUEIDENTIFIER,
  OPE_VIEW_ID BIGINT,
  OPE_MODEL_ID BIGINT,
  OPE_USER_ID INT,
  OPE_FLAGS INT,
  OPE_MAX_VALUE BIGINT,
  OPE_VALUE BIGINT,
  OPE_START_TIME DATETIME,
  OPE_END_TIME DATETIME,
  OPE_MESSAGE NVARCHAR(MAX),
  OPE_ADDITIONAL_DATA NVARCHAR(MAX),
  OPE_STATE NVARCHAR(MAX),
  OPE_LAST_POLLING_TIME DATETIME
);
ALTER TABLE PA_OPERATION_STATUS ADD PRIMARY KEY (OPE_ID);
CREATE INDEX PA_OPERATION_STATUS_SESSION_ID ON PA_OPERATION_STATUS (OPE_SESSION_ID);
CREATE INDEX PA_OPERATION_STATUS_START_TIME ON PA_OPERATION_STATUS (OPE_START_TIME);
ALTER TABLE PA_OPERATION_STATUS ADD CONSTRAINT PA_OPERATION_STATUS_USER FOREIGN KEY (OPE_USER_ID) REFERENCES PA_USER(USR_ID);

CREATE TABLE PA_SESSION (
  SES_ID BIGINT IDENTITY PRIMARY KEY,
  SES_SESSION_ID UNIQUEIDENTIFIER DEFAULT NEWID(),
  SES_CREATED_DATE DATETIME,
  SES_FINISHED_DATE DATETIME,
  SES_LAST_ACCESS DATETIME,
  SES_USER_ID INT,
  SES_USER_LOGON_NAME NVARCHAR(440),
  SES_USER_IP_ADDRESS NVARCHAR(MAX),
  SES_FLAGS INT,
  SES_CLIENT_VERSION NVARCHAR(MAX),
  SES_SERVER_VERSION NVARCHAR(MAX),
  SES_ADDITIONAL_DATA NVARCHAR(MAX),
  SES_AUTHENTICATION_METHOD INT
);
ALTER TABLE PA_SESSION ADD CONSTRAINT PA_SESSION_USER FOREIGN KEY (SES_USER_ID) REFERENCES PA_USER(USR_ID);
CREATE INDEX PA_SESSION_SESSION_ID ON PA_SESSION (SES_SESSION_ID);
CREATE INDEX PA_SESSION_USER_ID ON PA_SESSION (SES_USER_ID);
CREATE INDEX PA_SESSION_FINISHED_DATE ON PA_SESSION (SES_FINISHED_DATE);

CREATE TABLE PA_USER_ATTRIBUTE (
  UAT_ID BIGINT IDENTITY NOT NULL,
  UAT_USER_ID INT,
  UAT_MODEL_ID BIGINT,
  UAT_PROJECT_ID BIGINT,
  UAT_KEY NVARCHAR(440),
  UAT_VALUE SQL_VARIANT
);

ALTER TABLE PA_USER_ATTRIBUTE ADD PRIMARY KEY (UAT_ID);
ALTER TABLE PA_USER_ATTRIBUTE ADD CONSTRAINT PA_USER_ATTRIBUTE_MODEL FOREIGN KEY
	(UAT_MODEL_ID) REFERENCES PA_MODEL(MOD_ID);
CREATE INDEX PA_USER_ATTRIBUTE_MODEL_ID ON PA_USER_ATTRIBUTE (UAT_MODEL_ID);
ALTER TABLE PA_USER_ATTRIBUTE ADD CONSTRAINT PA_USER_ATTRIBUTE_USER FOREIGN KEY
	(UAT_USER_ID) REFERENCES PA_USER(USR_ID);
CREATE INDEX PA_USER_ATTRIBUTE_USER_ID ON PA_USER_ATTRIBUTE (UAT_USER_ID);
ALTER TABLE PA_USER_ATTRIBUTE ADD CONSTRAINT PA_USER_ATTRIBUTE_PROJECT FOREIGN KEY
	(UAT_PROJECT_ID) REFERENCES PA_PROJECT(PRO_ID);
CREATE INDEX PA_USER_ATTRIBUTE_PROJECT_ID ON PA_USER_ATTRIBUTE (UAT_PROJECT_ID);
CREATE INDEX PA_USER_ATTRIBUTE_KEY ON PA_USER_ATTRIBUTE (UAT_KEY);

CREATE TABLE PA_CONFIGURATION (
  CFG_ID INT IDENTITY NOT NULL,
  CFG_KEY NVARCHAR(440),
  CFG_VALUE SQL_VARIANT
);

ALTER TABLE PA_CONFIGURATION ADD PRIMARY KEY (CFG_ID);
CREATE INDEX PA_CONFIGURATION_KEY ON PA_CONFIGURATION (CFG_KEY);

CREATE TABLE PA_INTEGRATION_TABLE (
  ITA_ID BIGINT IDENTITY NOT NULL PRIMARY KEY,
  ITA_PROJECT_ID BIGINT,
  ITA_NAME NVARCHAR(440),
  ITA_DESCRIPTION NVARCHAR(MAX),
  ITA_CONFIGURATION NVARCHAR(MAX),
  ITA_FLAGS INT,
  ITA_CREATED_DATE DATETIME,
  ITA_CREATED_BY INT,
  ITA_LAST_MODIFIED_DATE DATETIME,
  ITA_LAST_MODIFIED_BY INT,
  ITA_LAST_IMPORT_DATE DATETIME,
  ITA_LAST_IMPORT_BY INT,
  ITA_LOG NVARCHAR(MAX),
  ITA_MAX_NUM_COLUMNS BIGINT,
  ITA_MAX_NUM_ROWS BIGINT,
  ITA_NUM_ROWS BIGINT
);

ALTER TABLE PA_INTEGRATION_TABLE ADD CONSTRAINT PA_INTEGRATION_TABLE_CREATED_BY FOREIGN KEY (ITA_CREATED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_INTEGRATION_TABLE ADD CONSTRAINT PA_INTEGRATION_TABLE_LAST_MODIFIED_BY FOREIGN KEY (ITA_LAST_MODIFIED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_INTEGRATION_TABLE ADD CONSTRAINT PA_INTEGRATION_TABLE_LAST_IMPORT_BY FOREIGN KEY (ITA_LAST_IMPORT_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_INTEGRATION_TABLE ADD CONSTRAINT PA_INTEGRATION_TABLE_PROJECT FOREIGN KEY (ITA_PROJECT_ID) REFERENCES PA_PROJECT(PRO_ID);
ALTER TABLE PA_INTEGRATION_TABLE ADD CONSTRAINT PA_INTEGRATION_TABLE_UNIQUE_NAME_IN_PROJECT UNIQUE (ITA_PROJECT_ID, ITA_NAME);
CREATE INDEX PA_INTEGRATION_TABLE_PROJECT_ID ON PA_INTEGRATION_TABLE (ITA_PROJECT_ID);
CREATE INDEX PA_INTEGRATION_TABLE_PROJECT_NAME_ID ON PA_INTEGRATION_TABLE (ITA_PROJECT_ID, ITA_NAME);
CREATE INDEX PA_INTEGRATION_TABLE_CREATED_BY ON PA_INTEGRATION_TABLE (ITA_CREATED_BY);

CREATE TABLE PA_SCRIPT (
  SCR_ID BIGINT IDENTITY NOT NULL PRIMARY KEY,
  SCR_GLOBAL_ID UNIQUEIDENTIFIER DEFAULT NEWID(),
  SCR_PROJECT_ID BIGINT,
  SCR_MODEL_ID BIGINT,
  SCR_USER_ID INT,
  SCR_CREATED_DATE DATETIME,
  SCR_CREATED_BY INT,
  SCR_LAST_MODIFIED_DATE DATETIME,
  SCR_LAST_MODIFIED_BY INT,
  SCR_TITLE NVARCHAR(440),
  SCR_TEXT NVARCHAR(MAX),
  SCR_DESCRIPTION NVARCHAR(MAX),
  SCR_SUPPORT_URL NVARCHAR(MAX),
  SCR_GROUP NVARCHAR(440),
  SCR_TAGS NVARCHAR(MAX),
  SCR_FLAGS INT,
  SCR_SCRIPTING_ENGINE_VERSION INT,
  SCR_LANGUAGE INT
);
ALTER TABLE PA_SCRIPT ADD CONSTRAINT PA_SCRIPT_PROJECT FOREIGN KEY (SCR_PROJECT_ID) REFERENCES PA_PROJECT(PRO_ID);
ALTER TABLE PA_SCRIPT ADD CONSTRAINT PA_SCRIPT_MODEL FOREIGN KEY (SCR_MODEL_ID) REFERENCES PA_MODEL(MOD_ID);
ALTER TABLE PA_SCRIPT ADD CONSTRAINT PA_SCRIPT_CREATED_BY FOREIGN KEY (SCR_CREATED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_SCRIPT ADD CONSTRAINT PA_SCRIPT_LAST_MODIFIED_BY FOREIGN KEY (SCR_LAST_MODIFIED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_SCRIPT ADD CONSTRAINT PA_SCRIPT_UNIQUE_GLOBAL_ID UNIQUE (SCR_GLOBAL_ID, SCR_PROJECT_ID, SCR_MODEL_ID, SCR_USER_ID);
CREATE INDEX PA_SCRIPT_PROJECT ON PA_SCRIPT (SCR_PROJECT_ID);
CREATE INDEX PA_SCRIPT_MODEL ON PA_SCRIPT (SCR_MODEL_ID);
CREATE INDEX PA_SCRIPT_USER ON PA_SCRIPT (SCR_USER_ID);

CREATE TABLE PA_SCRIPT_PARAMETER (
  SCP_ID BIGINT IDENTITY NOT NULL PRIMARY KEY,
  SCP_SCRIPT_ID BIGINT,
  SCP_TYPE INT,
  SCP_FLAGS INT,
  SCP_NAME NVARCHAR(440),
  SCP_DESCRIPTION NVARCHAR(MAX)
);
ALTER TABLE PA_SCRIPT_PARAMETER ADD CONSTRAINT PA_SCRIPT_PARAMETER_SCRIPT FOREIGN KEY (SCP_SCRIPT_ID) REFERENCES PA_SCRIPT(SCR_ID);
CREATE INDEX PA_SCRIPT_PARAMETER_SCRIPT ON PA_SCRIPT_PARAMETER (SCP_SCRIPT_ID);

CREATE TABLE PA_SCRIPT_LOG (
  SCL_ID BIGINT IDENTITY NOT NULL PRIMARY KEY,
  SCL_SCRIPT_ID BIGINT,
  SCL_TYPE INT,
  SCL_FLAGS INT,
  SCL_START_TIME DATETIME,
  SCL_END_TIME DATETIME,
  SCL_USER_ID INT,
  SCL_RESULT NVARCHAR(MAX),
  SCL_LOG NVARCHAR(MAX),
  SCL_ADDITIONAL_DATA NVARCHAR(MAX),
  SCL_OPERATION_ID BIGINT
);
ALTER TABLE PA_SCRIPT_LOG ADD CONSTRAINT PA_SCRIPT_LOG_USER_ID FOREIGN KEY (SCL_USER_ID) REFERENCES PA_USER(USR_ID);
CREATE NONCLUSTERED INDEX PA_SCRIPT_LOG_TYPE ON PA_SCRIPT_LOG (SCL_SCRIPT_ID, SCL_TYPE) INCLUDE (SCL_OPERATION_ID);
CREATE NONCLUSTERED INDEX PA_SCRIPT_LOG_END_TIME ON PA_SCRIPT_LOG (SCL_END_TIME);
CREATE NONCLUSTERED INDEX PA_SCRIPT_LOG_OPERATION_ID ON PA_SCRIPT_LOG (SCL_OPERATION_ID);

CREATE TABLE PA_UI_ELEMENT (
  UIE_ID BIGINT IDENTITY NOT NULL PRIMARY KEY,
  UIE_IDENTIFIER NVARCHAR(MAX),
  UIE_PROJECT_ID BIGINT,
  UIE_CREATED_DATE DATETIME,
  UIE_CREATED_BY INT,
  UIE_LAST_MODIFIED_DATE DATETIME,
  UIE_LAST_MODIFIED_BY INT,
  UIE_PROPERTIES NVARCHAR(MAX),
  UIE_NAME NVARCHAR(440),
  UIE_DESCRIPTION NVARCHAR(MAX),
  UIE_TYPE int
);
ALTER TABLE PA_UI_ELEMENT ADD CONSTRAINT PA_UI_ELEMENT_CREATED_BY_USER_ID FOREIGN KEY (UIE_CREATED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_UI_ELEMENT ADD CONSTRAINT PA_UI_ELEMENT_LAST_MODIFIED_BY_USER_ID FOREIGN KEY (UIE_LAST_MODIFIED_BY) REFERENCES PA_USER(USR_ID);
ALTER TABLE PA_UI_ELEMENT ADD CONSTRAINT PA_UI_ELEMENT_PROJECT FOREIGN KEY (UIE_PROJECT_ID) REFERENCES PA_PROJECT(PRO_ID);
CREATE INDEX PA_UI_ELEMENT_TYPE ON PA_UI_ELEMENT(UIE_TYPE);

CREATE TABLE PA_KEY_VALUE_STORE (
  KVS_ID BIGINT IDENTITY NOT NULL PRIMARY KEY,
  KVS_TYPE INT,
  KVS_PROJECT_ID BIGINT,
  KVS_KEY NVARCHAR(440),
  KVS_VALUE NVARCHAR(MAX)
);
ALTER TABLE PA_KEY_VALUE_STORE ADD CONSTRAINT PA_KEY_VALUE_STORE_PROJECT FOREIGN KEY (KVS_PROJECT_ID) REFERENCES PA_PROJECT(PRO_ID);

DECLARE	@administratorRoleId BIGINT; INSERT INTO PA_ROLE (ROL_NAME, ROL_FLAGS) VALUES ('Administrator', 3); SELECT @administratorRoleId = SCOPE_IDENTITY();
DECLARE	@modelCreatorRoleId BIGINT; INSERT INTO PA_ROLE (ROL_NAME, ROL_FLAGS) VALUES ('ModelCreator', 1); SELECT @modelCreatorRoleId = SCOPE_IDENTITY();
DECLARE	@runScriptsRoleId BIGINT; INSERT INTO PA_ROLE (ROL_NAME, ROL_FLAGS) VALUES ('RunScripts', 1); SELECT @runScriptsRoleId = SCOPE_IDENTITY();
DECLARE	@designerRoleId BIGINT; INSERT INTO PA_ROLE (ROL_NAME, ROL_FLAGS) VALUES ('Designer', 2); SELECT @designerRoleId = SCOPE_IDENTITY();
DECLARE	@analyzerRoleId BIGINT; INSERT INTO PA_ROLE (ROL_NAME, ROL_FLAGS) VALUES ('Analyzer', 2); SELECT @analyzerRoleId = SCOPE_IDENTITY();
DECLARE	@viewerRoleId BIGINT; INSERT INTO PA_ROLE (ROL_NAME, ROL_FLAGS) VALUES ('Viewer', 2); SELECT @viewerRoleId = SCOPE_IDENTITY();
DECLARE	@evaluatorRoleId BIGINT; INSERT INTO PA_ROLE (ROL_NAME, ROL_FLAGS, ROL_MAX_NUM_MODELS, ROL_MODEL_MAX_NUM_EVENTS, ROL_MODEL_MAX_NUM_CASE_ATTRIBUTE_TYPES, ROL_MODEL_MAX_NUM_EVENT_ATTRIBUTE_TYPES, ROL_MAX_NUM_INTEGRATION_TABLES, ROL_INTEGRATION_TABLE_MAX_NUM_COLUMNS, ROL_INTEGRATION_TABLE_MAX_NUM_ROWS) VALUES ('Evaluator', 1, 10, 1000, 1000, 1000, 10, 1000, 1000); SELECT @evaluatorRoleId = SCOPE_IDENTITY(); 

DECLARE	@administratorUserId BIGINT; INSERT INTO PA_USER (USR_LOGON_NAME, USR_PASSWORD_HASH) VALUES ('Administrator', 'JoqKyoDqMicuu0narinCyvHQ7Z2PuANZrnJEiqHsORzd3xzBDAutTdFE6EeqrHpr'); SELECT @administratorUserId = SCOPE_IDENTITY();

DECLARE	@genericReadPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('GenericRead'); SELECT @genericReadPermissionId = SCOPE_IDENTITY();
DECLARE	@genericWritePermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('GenericWrite'); SELECT @genericWritePermissionId = SCOPE_IDENTITY();
DECLARE	@deleteModelPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('DeleteModel'); SELECT @deleteModelPermissionId = SCOPE_IDENTITY();
DECLARE	@createModelPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('CreateModel'); SELECT @createModelPermissionId = SCOPE_IDENTITY();
DECLARE	@resetDatabasePermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ResetDatabase'); SELECT @resetDatabasePermissionId = SCOPE_IDENTITY();
DECLARE	@filteringPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('Filtering'); SELECT @filteringPermissionId = SCOPE_IDENTITY();
DECLARE	@manageUsersPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ManageUsers'); SELECT @manageUsersPermissionId = SCOPE_IDENTITY();
DECLARE	@sqlImportPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('SqlImport'); SELECT @sqlImportPermissionId = SCOPE_IDENTITY();
DECLARE	@manageOperationsPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ManageOperations'); SELECT @manageOperationsPermissionId = SCOPE_IDENTITY();
DECLARE	@manageViewsPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ManageViews'); SELECT @manageViewsPermissionId = SCOPE_IDENTITY();
DECLARE	@manageReportsPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ManageReports'); SELECT @manageReportsPermissionId = SCOPE_IDENTITY();
DECLARE	@manageIdeasPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ManageIdeas'); SELECT @manageIdeasPermissionId = SCOPE_IDENTITY();
DECLARE	@manageProjectPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ManageProject'); SELECT @manageProjectPermissionId = SCOPE_IDENTITY();
DECLARE	@manageIntegrationsPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ManageIntegrations'); SELECT @manageIntegrationsPermissionId = SCOPE_IDENTITY();
DECLARE	@runScriptsPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('RunScripts'); SELECT @runScriptsPermissionId = SCOPE_IDENTITY();
DECLARE	@manageScriptsPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('ManageScripts'); SELECT @manageScriptsPermissionId = SCOPE_IDENTITY();
DECLARE	@editDashboardsPermissionId BIGINT; INSERT INTO PA_PERMISSION (PER_NAME) VALUES ('EditDashboards'); SELECT @editDashboardsPermissionId = SCOPE_IDENTITY();

INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @genericReadPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @genericWritePermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @deleteModelPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @createModelPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @resetDatabasePermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @filteringPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @manageUsersPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @manageOperationsPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @manageViewsPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @manageReportsPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @manageIdeasPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @manageProjectPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @manageIntegrationsPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @manageScriptsPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@administratorRoleId, @editDashboardsPermissionId);

INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@designerRoleId, @genericReadPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@designerRoleId, @genericWritePermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@designerRoleId, @createModelPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@designerRoleId, @filteringPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@designerRoleId, @editDashboardsPermissionId);

INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@analyzerRoleId, @genericReadPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@analyzerRoleId, @filteringPermissionId);
INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@analyzerRoleId, @editDashboardsPermissionId);

INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@viewerRoleId, @genericReadPermissionId);

INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@modelCreatorRoleId, @createModelPermissionId);

INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@evaluatorRoleId, @createModelPermissionId);

INSERT INTO PA_ROLE_PERMISSION (ROP_ROLE_ID, ROP_PERMISSION_ID) VALUES (@runScriptsRoleId, @runScriptsPermissionId);

INSERT INTO PA_USER_ROLE (URO_USER_ID, URO_ROLE_ID) VALUES (@administratorUserId, @administratorRoleId);
INSERT INTO PA_USER_ROLE (URO_USER_ID, URO_ROLE_ID) VALUES (@administratorUserId, @runScriptsRoleId);

INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('DatabaseId', CONVERT(NVARCHAR(255), NEWID()));
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('DatabaseBulkCopyTimeout', '600');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('DatabaseBulkCopyBatchSize', '5000');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('DatabaseAvailabilityMode', '0');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SessionIdleTimeout', '3600');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SessionMaximumDuration', '86400');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('DeleteModelsInRecycleBinOnStartup', '0');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SandboxDatabaseConnectionString', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SandboxDatabaseBulkCopyTimeout', '600');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SandboxDatabaseBulkCopyBatchSize', '5000');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('StartupModelLoadingMaxParallelism', '2');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('AllowExternalDatasources', 'true');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('ServiceProviderLocation', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SAMLMetadataUrl', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SAMLUserIdAttribute', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SAMLEncryptionCertificate', null);
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SAMLSigningCertificate', null);
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('DefaultUiLanguage', null);
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('NumberOfParallelModelReaders', '4');

/*
  Disable non-temporary ETL target table by default.
*/
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('AllowNonTemporaryETLTargetTable', 'false');

/*
  Authentication settings
*/
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('AuthenticationMethod', '1');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('LDAPConnectionString', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('LDAPUserFilter', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('LDAPUserSearchBase', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('LDAPUserIdAttributeName', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('LDAPServerUserName', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('LDAPServerPassword', '');

/*
  SMTP mail server settings
*/
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SmtpServer', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SmtpPort', '25');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SmtpAuthenticationUsername', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SmtpAuthenticationPassword', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SmtpEnableSSL', 'false');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SmtpFromAddress', '');

/*
  Connection strings to external systems.
*/
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('RedshiftConnectionString', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SqlServerConnectionString', '');
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('SnowflakeConnectionString', '');

/*
	QueryTimeout value for api/expression and api/expression/query in seconds
*/
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('QueryTimeout', '300');

/*
  Defines to which datasource the datatables are created by default
*/
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('DefaultDataSource', 'Snowflake');

SET IDENTITY_INSERT PA_PERMISSION ON;

DECLARE @dbVersion SQL_VARIANT
SET @dbVersion = '95'

INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('DatabaseVersion', @dbVersion);
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('InitializationScriptDatabaseVersion', @dbVersion);
INSERT INTO PA_CONFIGURATION (CFG_KEY, CFG_VALUE) VALUES ('MinimumDatabaseVersion', @dbVersion);



/*
  Enable better non-blocking concurrent connection handling.

  NOTE: 
    These commands must be run using implicit transactions. 
    Otherwise we will get: "ALTER DATABASE statement not allowed within multi-statement transaction"-exception.
*/
DECLARE @databaseName NVARCHAR(MAX)
SELECT @databaseName = DB_NAME();
DECLARE @userAccess INT;
DECLARE @recoveryModel INT;
DECLARE @isReadCommittedSnapshotOn INT;
DECLARE @snapshotIsolationState INT;
SELECT @userAccess = user_access, @isReadCommittedSnapshotOn = is_read_committed_snapshot_on, @recoveryModel = recovery_model, @snapshotIsolationState = snapshot_isolation_state FROM sys.databases WHERE name = @databaseName;

IF ((@userAccess <> 0) OR (@recoveryModel <> 3) OR (@isReadCommittedSnapshotOn <> 1) OR (@snapshotIsolationState <> 1)) BEGIN 
  DECLARE @dynamicSql NVARCHAR(MAX);
  SET @dynamicSql = '
  ALTER DATABASE [' + @databaseName + '] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
  ALTER DATABASE [' + @databaseName + '] SET ALLOW_SNAPSHOT_ISOLATION ON;
  ALTER DATABASE [' + @databaseName + '] SET READ_COMMITTED_SNAPSHOT ON;
  ALTER DATABASE [' + @databaseName + '] SET RECOVERY SIMPLE;
  ALTER DATABASE [' + @databaseName + '] SET MULTI_USER;
  '
  EXEC (@dynamicSql) 
END;
GO
